#!/usr/bin/env python

#
# Find all images below a directory and create a matlab file with
# image names and data.
#
# Requires EXIF.py: http://home.cfl.rr.com/genecash/digital_camera.html
# 
# You can remove these requirements if you only need file names and not
# size or other EXIF info.  The script uses PIL for image dimensions and 
# image format and it uses EXIF.py for other EXIF data (f-stop, focal length).
#
# Last Modified: 6/7/2006
import sys
import os.path

# Don't use EXIF
use_meta = True

if use_meta:
	import EXIF


extensions = ['.jpg', '.jpeg', '.gif', '.tif', '.tiff', '.png', '.hdr']
formats    = ['JPEG', 'JPEG',  'GIF',  'TIFF', 'TIFF',  'PNG', 'HDR']
format_dict = dict(zip(extensions,formats))

#
# Process command line arguments
#
def processArgs(argv):
	argc = len(argv)
	if argc < 3:
		print "Usage: find_images.py <directory> <matlab output file>"
		sys.exit()

	args = map(lambda s: s.strip(), argv[1:])
	
	# Make sure directory exists
	if not os.path.exists(args[0]):
		print 'Directory "%s" does not exist.' % args[0]
		sys.exit()
	
	# Make sure file name has .m as extension
	(root, ext) = os.path.splitext(args[1])
	if ext != '.m':
		args[1] = root + '.m'
		print "changed output file to %s." % args[1]

	return tuple(args)

#
# Add filename to list if the extension is in the list
# of extensions specified above.
#
def getAllImages(images, root, names):
	for name in names:
		(base, ext) = os.path.splitext(name)
		if ext.lower() in extensions:
			images.append(os.path.join(root,name))
#
#
#
def main(argv):
	(source_dir, m_file) = processArgs(argv)

	images = []
	os.path.walk(source_dir, getAllImages, images)

	# Print image names to matlab file
	fd = open(m_file,'w')
	(base, ext) = os.path.splitext(m_file)
	
	fd.write('function names = %s()\n\n' % base)

	if use_meta:
		fd.write('data = cell(%d,7);\n' % len(images))
	else:
		fd.write('data = cell(%d,1);\n' % len(images))			

	
	image_ix = 1;
	for image in images:
		if use_meta:
			exif_im = open(image)
			tags = EXIF.process_file(exif_im)
			# Default settings
			ydim = xdim = 0
			fnumber = 'f/0'
			shutter = '0'
			focal = '0'

			# Get size
			if tags.has_key('EXIF ExifImageLength'):			
				ydim = int(tags['EXIF ExifImageLength'].printable)

			if tags.has_key('EXIF ExifImageWidth'):			
				xdim = int(tags['EXIF ExifImageWidth'].printable)

			# Get format
			(root,ext) = os.path.splitext(image)
			format = format_dict.get(ext.lower(),'unknown')

			# Get F-number
			if tags.has_key('EXIF FNumber'):
				fnumber = 'f/%.1f' % eval(tags['EXIF FNumber'].printable + '.0')
				# Prune .0 from end
				if fnumber[-2:] == '.0':
					fnumber = fnumber[:-2]

			if tags.has_key('EXIF ExposureTime'):
				shutter = tags['EXIF ExposureTime'].printable
						
			if tags.has_key('EXIF FocalLength'):
				focal = tags['EXIF FocalLength'].printable

			line = "data(%d,:) = {'%s','%s', %d, %d, '%s', '%s', '%s'};\n" % \
				(image_ix, image, format, xdim, ydim, focal, shutter, fnumber)
			
			exif_im.close()
				
		else:
			line = "data(%d) = {'%s'};\n" % (image_ix, image)

		fd.write(line)
		image_ix += 1

	# Convert cell array to struct array
	if use_meta:
		fd.write("\nnames = cell2struct(data, {'path', 'type', 'xdim', 'ydim', 'focal', 'shutter', 'fnumber'}, 2);\n")
	else:
		fd.write("\nnames = cell2struct(data, {'path'}, 2);\n")

	fd.close()
	
if __name__ == "__main__":
	main(sys.argv)
